@@ -369,11 +369,11 @@ function hardware_profile_list($con, $conf){
     return $json;
 }
 
-function hardware_profile_remove_hdd_in_slot($physdrives, $slot_num){
+function hardware_profile_remove_hdd_in_slot($physdrives, $hdd){
     $hdd_indexes = array_keys($physdrives);
     foreach($hdd_indexes as &$hdd_index){
         $one_hdd = $physdrives[$hdd_index];
-        if($one_hdd["slot"] == $slot_num){
+        if($one_hdd["slot"] == $hdd["slot"] && $one_hdd["controller"] == $hdd["controller"] && $one_hdd["enclosure"] == $hdd["enclosure"]){
 //            print("Found hdd to remove in index $hdd_index\n");
             unset($physdrives[$hdd_index]);
             return $physdrives;
@@ -382,6 +382,52 @@ function hardware_profile_remove_hdd_in_slot($physdrives, $slot_num){
     return $physdrives;
 }
 
+function search_raid_cli($con, $conf, $machine){
+    $json["status"] = "success";
+    $json["message"] = "Successfuly queried API.";
+
+    $cmd = "which storcli64";
+    $exe_path = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+
+    if($exe_path == "/usr/bin/storcli64"){
+        $cmd = "storcli64 /call/eall/sall show | grep 'No Controller found'";
+        $no_ctrl = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+        if($no_ctrl != 'Description = No Controller found'){
+            return "storcli64";
+        }
+    }
+
+    $cmd = "which storcli_x64";
+    $exe_path = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+
+    if($exe_path == "/usr/bin/storcli_x64"){
+        $cmd = "storcli_x64 /call/eall/sall show | grep 'No Controller found'";
+        $no_ctrl = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+        if($no_ctrl != 'Description = No Controller found'){
+            return "storcli_x64";
+        }
+    }
+
+    $cmd = "which storcli";
+    $exe_path = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+
+    if($exe_path == "/usr/bin/storcli"){
+        $cmd = "storcli /call/eall/sall show | grep 'No Controller found'";
+        $no_ctrl = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+        if($no_ctrl != 'Description = No Controller found'){
+            return "storcli";
+        }
+    }
+
+    $cmd = "which megacli";
+    $exe_path = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+
+    if($exe_path == "/usr/sbin/megacli"){
+        return "megacli";
+    }
+    return FALSE;
+}
+
 function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_name, $debug="no"){
     if($debug == "yes"){
         $debug_megacli_apply = "yes";
@@ -443,8 +489,19 @@ function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_
         }
     }
 
+    $cli_path = search_raid_cli($con, $conf, $machine);
+    if($cli_path == FALSE){
+        $json["status"] = "error";
+        $json["message"] = "Cannot find storcli/megacli.";
+        return $json;
+    }
+
     # Get the enclosure number
-    $cmd = "megacli -EncInfo -aALL -NoLog | grep 'Device ID' | cut -d: -f2 | cut -d' ' -f2";
+    if($cli_path == "megacli"){
+        $cmd = "megacli -EncInfo -aALL -NoLog | grep 'Device ID' | cut -d: -f2 | cut -d' ' -f2";
+    }else{
+        $cmd = "$cli_path /call show | grep -A6 'Enclosure LIST' | tail -n1 | cut -d' ' -f1";
+    }
     $ret = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
     $enclosure_number = $ret;
 
@@ -458,7 +515,11 @@ function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_
     }
 
     # Clear all foreign RAID
-    $cmd = "megacli -CfgForeign -Clear -a0";
+    if($cli_path == "megacli"){
+        $cmd = "megacli -CfgForeign -Clear -a0";
+    }else{
+        $cmd = "$cli_path /c0/eall/sall delete jbod";
+    }
     if($debug_megacli_apply == "yes"){
         $json["data"][] = $cmd;
     }else{
@@ -466,7 +527,11 @@ function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_
     }
 
     # Delete any existing virtual disk
-    $cmd = "megacli -CfgLdDel -LALL -a0";
+    if($cli_path == "megacli"){
+        $cmd = "megacli -CfgLdDel -LALL -a0";
+    }else{
+        $cmd = "$cli_path /call /vall del";
+    }
     if($debug_megacli_apply == "yes"){
         $json["data"][] = $cmd;
     }else{
@@ -475,8 +540,14 @@ function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_
 
     # Make all HDD as "Unconfigured, Good"
     foreach($machine["physbd"] as &$hdd){
+        $controller = $hdd["controller"];
+        $enclosure = $hdd["enclosure"];
         $slot = $hdd["slot"];
-        $cmd = "megacli -PDMakeGood -PhysDrv[$enclosure_number:$slot] -force -a0";
+        if($cli_path == "megacli"){
+            $cmd = "megacli -PDMakeGood -PhysDrv[$enclosure_number:$slot] -force -a0";
+        }else{
+            $cmd = "$cli_path /c$controller/e$enclosure/s$slot set good";
+        }
         if($debug_megacli_apply == "yes"){
             $json["data"][] = $cmd;
         }else{
@@ -494,10 +565,10 @@ function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_
         foreach($hdd_indexes as &$hdd_index){
             $one_hdd = $machine_physbd_tmp[$hdd_index];
             $hdd_size_gb = floor($one_hdd["size"] / (1024 * 1024 * 1024));
-            $hdd_slot = $one_hdd["slot"];
+            $hdd_slot = $one_hdd;
             if($hdd_size_gb >= $layout["size_min"] && $hdd_size_gb <= $layout["size_max"]){
 //                echo "Hdd in slot " . $one_hdd["slot"] . " matches.\n";
-                $hdd_slots[] = $one_hdd["slot"];
+                $hdd_slots[] = $one_hdd;
 //                unset($machine_physbd_tmp[$hdd_index]);
             }
         }
@@ -511,29 +582,56 @@ function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_
 //        print_r($machine_physbd_tmp);
         switch($raid_type){
         case "JBOD":
-            $megacli_cfg = "megacli -PDMakeJBOD -PhysDrv [";
+            $raid_megacli_cfg_opt = "";
+            if($cli_path == "megacli"){
+                $megacli_cfg = "megacli -PDMakeJBOD -PhysDrv [";
+            }else{
+                $megacli_cfg = "";
+            }
             $cnt = 0;
             foreach($hdd_slots as &$one_slot){
                 if($cnt != 0){
-                    $megacli_cfg .= ",";
+                    if($cli_path == "megacli"){
+                        $megacli_cfg .= ",";
+                    }
+                }
+                $my_controller = $one_slot["controller"];
+                $my_enclosure = $one_slot["enclosure"];
+                $my_slot = $one_slot["slot"];
+                if($cli_path == "megacli"){
+                    $megacli_cfg .= "$my_enclosure:$my_slot";
+                }else{
+                    $megacli_cfg .= "$cli_path /c$my_controller/e$my_enclosure/s$my_slot set jbod";
                 }
-                $megacli_cfg .= "$enclosure_number:$one_slot";
                 $machine_physbd_tmp = hardware_profile_remove_hdd_in_slot($machine_physbd_tmp, $one_slot);
                 $cnt+=1;
                 if($cnt >= $layout["num_max"]){
                     break;
                 }
             }
-            $megacli_cfg .= "] -a0";
+            if($cli_path == "megacli"){
+                $megacli_cfg .= "] -a0";
+            }
             break;
         case "0":
-            $megacli_cfg = "megacli -CfgLdAdd -r0 [";
             $cnt = 0;
+            $raid_megacli_cfg_opt = "";
             foreach($hdd_slots as &$one_slot){
                 if($cnt != 0){
-                    $megacli_cfg .= ",";
+                    if($cli_path == "megacli"){
+                        $raid_megacli_cfg_opt .= ",";
+                    }else{
+                        $raid_megacli_cfg_opt .= "-";
+                    }
+                }
+                $my_controller = $one_slot["controller"];
+                $my_enclosure = $one_slot["enclosure"];
+                $my_slot = $one_slot["slot"];
+                if($cli_path == "megacli"){
+                    $raid_megacli_cfg_opt .= "$my_enclosure:$my_slot";
+                }else{
+                    $raid_megacli_cfg_opt .= "$my_slot";
                 }
-                $megacli_cfg .= "$enclosure_number:$one_slot";
 //                print("Removing slot $one_slot\n");
                 $machine_physbd_tmp = hardware_profile_remove_hdd_in_slot($machine_physbd_tmp, $one_slot);
                 $cnt+=1;
@@ -541,23 +639,48 @@ function hardware_profile_apply_megacli_raid($con, $conf, $machine_id, $profile_
                     break;
                 }
             }
-            $megacli_cfg .= "] $options -a0";
+            if($cli_path == "megacli"){
+                $megacli_cfg = "megacli -CfgLdAdd -r0 [";
+            }else{
+                $megacli_cfg = "$cli_path /call add vd type=r0 drives=$my_enclosure:$raid_megacli_cfg_opt";
+            }
+            if($cli_path == "megacli"){
+                $megacli_cfg .= "] $options -a0";
+            }
             break;
         case "1":
-            $megacli_cfg = "megacli -CfgLdAdd -r1 [";
             $cnt = 0;
+            $raid_megacli_cfg_opt = "";
             foreach($hdd_slots as &$one_slot){
                 if($cnt != 0){
-                    $megacli_cfg .= ",";
+                    if($cli_path == "megacli"){
+                        $raid_megacli_cfg_opt .= ",";
+                    }else{
+                        $raid_megacli_cfg_opt .= "-";
+                    }
+                }
+                $my_controller = $one_slot["controller"];
+                $my_enclosure = $one_slot["enclosure"];
+                $my_slot = $one_slot["slot"];
+                if($cli_path == "megacli"){
+                    $raid_megacli_cfg_opt .= "$my_enclosure:$my_slot";
+                }else{
+                    $raid_megacli_cfg_opt .= "$my_slot";
                 }
-                $megacli_cfg .= "$enclosure_number:$one_slot";
                 $machine_physbd_tmp = hardware_profile_remove_hdd_in_slot($machine_physbd_tmp, $one_slot);
                 $cnt+=1;
                 if($cnt >= $layout["num_max"]){
                     break;
                 }
             }
-            $megacli_cfg .= "] $options -a0";
+            if($cli_path == "megacli"){
+                $megacli_cfg = "megacli -CfgLdAdd -r1 [";
+            }else{
+                $megacli_cfg = "$cli_path /call add vd type=r1 drives=$my_enclosure:$raid_megacli_cfg_opt";
+            }
+            if($cli_path == "megacli"){
+                $megacli_cfg .= "] $options -a0";
+            }
             break;
         case "10":
             $megacli_cfg = "megacli -CfgSpanAdd -r10 ";
@@ -607,8 +730,19 @@ function hardware_profile_apply_megacli_reset_raid($con, $conf, $machine_id){
     }
     $machine = $ret["data"];
 
+    $cli_path = search_raid_cli($con, $conf, $machine);
+    if($cli_path == FALSE){
+        $json["status"] = "error";
+        $json["message"] = "Cannot find storcli/megacli.";
+        return $json;
+    }
+
     # Clear all foreign RAID
-    $cmd = "megacli -CfgForeign -Clear -a0";
+    if($cli_path == "megacli"){
+        $cmd = "megacli -CfgForeign -Clear -a0";
+    }else{
+        $cmd = "$cli_path /c0/eall/sall delete jbod";
+    }
     if($debug_megacli_apply == "yes"){
         print($cmd."\n");
     }else{
@@ -616,7 +750,11 @@ function hardware_profile_apply_megacli_reset_raid($con, $conf, $machine_id){
     }
 
     # Delete any existing virtual disk
-    $cmd = "megacli -CfgLdDel -LALL -a0";
+    if($cli_path == "megacli"){
+        $cmd = "megacli -CfgLdDel -LALL -a0";
+    }else{
+        $cmd = "$cli_path /call /vall del";
+    }
     if($debug_megacli_apply == "yes"){
         print($cmd."\n");
     }else{
@@ -624,22 +762,32 @@ function hardware_profile_apply_megacli_reset_raid($con, $conf, $machine_id){
     }
 
     # Do a CfgClr
-    $cmd = "megacli -CfgClr -Force -aALL";
-    if($debug_megacli_apply == "yes"){
-        print($cmd."\n");
-    }else{
-        $ret = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+    if($cli_path == "megacli"){
+        $cmd = "megacli -CfgClr -Force -aALL";
+        if($debug_megacli_apply == "yes"){
+            print($cmd."\n");
+        }else{
+            $ret = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
+        }
     }
 
     # Get the enclosure number
-    $cmd = "megacli -EncInfo -aALL -NoLog | grep 'Device ID' | cut -d: -f2 | cut -d' ' -f2";
+    if($cli_path == "megacli"){
+        $cmd = "megacli -EncInfo -aALL -NoLog | grep 'Device ID' | cut -d: -f2 | cut -d' ' -f2";
+    }else{
+        $cmd = "$cli_path /call show | grep -A6 'Enclosure LIST' | tail -n1 | cut -d ' ' -f1";
+    }
     $ret = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
     $enclosure_number = $ret;
 
     # Make all HDD as "Unconfigured, Good"
     foreach($machine["physbd"] as &$hdd){
         $slot = $hdd["slot"];
-        $cmd = "megacli -PDMakeGood -PhysDrv[$enclosure_number:$slot] -force -a0";
+        if($cli_path == "megacli"){
+            $cmd = "megacli -PDMakeGood -PhysDrv[$enclosure_number:$slot] -force -a0";
+        }else{
+            $cmd = "$cli_path /call/eall/s$slot set good";
+        }
         if($debug_megacli_apply == "yes"){
             print($cmd."\n");
         }else{
