@@ -255,7 +255,7 @@ function perform_ipmitool_cmd($con, $conf, $machine_id, $ipmi_username, $ipmi_pa
         }
     }
     if($conf["hpe"]["live_image_install_hpe"] == "1"){
-        if($machine["product_name"] == "ProLiant DL385 Gen10" || $machine["product_name"] == "ProLiant DL385 Gen10 Plus"){
+        if($machine["product_name"] == "ProLiant DL385 Gen10" || $machine["product_name"] == "ProLiant DL385 Gen10 Plus" || $machine["product_name"] == "ProLiant DL345 Gen11"){
             $cmd = "hponcfg -f /etc/oci/hpe-activate-ipmi-over-lan.xml";
             $json["message"] .= "\n$cmd";
             send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
@@ -287,7 +287,7 @@ function perform_ipmitool_cmd($con, $conf, $machine_id, $ipmi_username, $ipmi_pa
     $out = send_ssh_cmd($conf, $con, $machine["ipaddr"], $cmd);
     $json["message"] .= "\n$out";
 
-    if($machine["ipmi_addr"] != $ipmi_addr && ($machine["product_name"] == "ProLiant DL385 Gen10" || $machine["product_name"] == "ProLiant DL385 Gen10 Plus")){
+    if($machine["ipmi_addr"] != $ipmi_addr && ($machine["product_name"] == "ProLiant DL385 Gen10" || $machine["product_name"] == "ProLiant DL385 Gen10 Plus" || $machine["product_name"] == "ProLiant DL345 Gen11")){
         # Enable the IPMI over LAN, which is off by default on these machines
         $cmd = "echo '<RIBCL VERSION=\\\"2.0\\\"><LOGIN USER_LOGIN=\\\"admin\\\" PASSWORD=\\\"password\\\"><RIB_INFO mode=\\\"write\\\"><MOD_GLOBAL_SETTINGS><IPMI_DCMI_OVER_LAN_ENABLED VALUE=\\\"Y\\\"/></MOD_GLOBAL_SETTINGS></RIB_INFO></LOGIN></RIBCL>' >set-ipmi-on.xml ; hponcfg -f set-ipmi-on.xml ; rm set-ipmi-on.xml";
         $json["message"] .= "\n$cmd";
@@ -307,9 +307,86 @@ function perform_ipmitool_cmd($con, $conf, $machine_id, $ipmi_username, $ipmi_pa
         $json["message"] .= "\n$out";
     }
 
+    if($machine["ipmi_addr"] != $ipmi_addr && $machine["product_name"] == "ProLiant DL345 Gen11"){
+        # Add an SMP community so that we can monitor the system NVMe RAID of these machines
+        $cmd = "oci-ilorest " . $machine["serial"] . 'set ReadCommunities=["snmp_read_string","",""] --selector=HpeiLOSnmpService.v2_3_0 --commit';
+        $output = array();
+        $return_var = 0;
+        exec($cmd, $output, $return_var);
+    }
+
+#    if($machine["system_manufacturer"] == "HPE"){
+#        # Enable IPMI over LAN
+#        $cmd = "oci-ilorest " . $machine["serial"] . "set --selector=NetworkProtocol IPMI/ProtocolEnabled=True --commit";
+#        $output = array();
+#        $return_var = 0;
+#        exec($cmd, $output, $return_var);
+#
+#        # Install iLO's license
+#        if ($conf["hpe"]["hpe_ilo_install_license"] == "1") {
+#            $cmd = "oci-ilorest " . $machine["serial"] . "ilolicense " . $conf["hpe"]["hpe_ilo_license"];
+#            $output = array();
+#            $return_var = 0;
+#            exec($cmd, $output, $return_var);
+#        }
+#
+#        # Reset so it takes the new IP
+#        $cmd = "oci-ilorest " . $machine["serial"] . "iloreset";
+#        $output = array();
+#        $return_var = 0;
+#        exec($cmd, $output, $return_var);
+#    }
+
     return $json;
 }
 
+function machine_activate_ipmi_over_lan($con, $conf, $machine){
+    switch($machine["system_manufacturer"]){
+    case "Dell Inc.":
+    case "Dell":
+        send_ssh_cmd($conf, $con, $machine["ipaddr"], "racadm set iDRAC.IPMILan.Enable Enabled");
+        break;
+    case "HPE":
+        $cmd = "oci-ilorest" . $machine["serial"] . "set --selector NetworkProtocol IPMI/ProtocolEnabled=True";
+        $output = array();
+        $return_var = 0;
+        exec($cmd, $output, $return_var);
+        break;
+    default:
+        break;
+    }
+}
+
+# Adopting an IPMI means using its IPMI username,
+# but set a random password, and leave it run over DHCP,
+# then save its IPMI password to a password vault if
+# the plugin is activated.
+function machine_ipmi_adopt($con, $conf, $machine){
+    $json["status"] = "success";
+    $json["message"] = "Successfuly queried API.";
+    if($machine["product_name"] == "Lenovo" || $machine["product_name"] == "LENOVO"){
+        # This produces a 16 char long pass
+        $num_bytes = 12;
+    }else{
+        # This produces a 12 char long pass
+        $num_bytes = 9;
+    }
+    # Get the current IPMI username for user 2.
+    $ipmi_username = send_ssh_cmd($conf, $con, $machine["ipaddr"], "ipmitool user list 1 2>/dev/null | grep ^2 | awk '{print \\$2}'");
+    # Generate a new IPMI password...
+    $ipmi_password = base64_encode(openssl_random_pseudo_bytes($num_bytes, $crypto_strong));
+    # and imediatelly set it with ipmitool.
+    send_ssh_cmd($conf, $con, $machine["ipaddr"], "ipmitool user set password 2 $ipmi_password");
+
+    machine_activate_ipmi_over_lan($con, $conf, $machine);
+
+    # Update the db with the new info...
+    $q = "UPDATE machines SET ipmi_username='$ipmi_username', ipmi_password='$ipmi_password', ipmi_addr='". $machine["ipmi_detected_ip"] ."', ipmi_use='yes', ipmi_call_chassis_bootdev='yes' WHERE id='" . $machine["id"] . "'";
+    $r = mysqli_query($con, $q);
+    # ... as well as the password vault.
+    machine_save_ipmi_pass($con, $conf, $machine);
+}
+
 function automatic_ipmi_assign($con, $conf, $machine_id){
     $q = "SELECT * FROM machines WHERE id='$machine_id'";
     $r = mysqli_query($con, $q);
@@ -365,6 +442,7 @@ function automatic_ipmi_assign($con, $conf, $machine_id){
                 }
             }
 
+
             if($matched_network != "no"){
                 # At this point, we know the machine should be having its IPMI
                 # address on $matched_network. Let's get some properties of it.
@@ -477,50 +555,54 @@ function automatic_ipmi_assign($con, $conf, $machine_id){
                     $ipmi_netmask = "0.0.0.0";
                     break;
                 }
-                $ret = get_ip_of_machine_on_network($con, $conf, $matched_network["id"], $machine_id);
-                if($ret["status"] != "success"){
-                    # If we don't have an IP on this network, there's 2 cases:
-                    # 1/ the machine never had its IPMI set and recorded in the DB.
-                    # 2/ the machine already was in an IPMI network, but that's the wrong one.
-                    # We simply get rid of all IPMI IPs for that machine, so we get back on case 1/.
-                    $qcheckwrong = "SELECT * FROM ips WHERE machine='".$machine["id"]."' AND usefor='ipmi'";
-                    $rcheckwrong = mysqli_query($con, $qcheckwrong);
-                    $ncheckwrong = mysqli_num_rows($rcheckwrong);
-                    if($ncheckwrong >= 0){
-                        $q_remove_ip = "DELETE FROM ips WHERE machine='".$machine["id"]."' AND usefor='ipmi'";
-                        $r_remove_ip = mysqli_query($con, $q_remove_ip);
+                if($network["ipmi_mode"] == "externaldhcp"){
+                    # If we have this, then the server should have been already set.
+                    if($machine["ipmi_addr"] == $machine["ipmi_detected_ip"]){
+                        return;
                     }
-
-                    $ret = reserve_ip_address($con, $conf, $net_id, $machine["id"], "ipmi");
-                    if($ret["status"] != "success"){
-                        print($ret["message"]);
-                    }
-                    $ret = get_ip_of_machine_on_network($con, $conf, $matched_network["id"], $machine_id);
-                    # Since we had to reserve a new IP, this means we got to do the IPMI setup.
-                    $do_update = "yes";
+                    $ret = machine_ipmi_adopt($con, $conf, $machine);
                 }else{
-                    # As an IP address was already reserved, it means we don't need to do the setup.
-                    $do_update = "no";
-                }
-                if($ret["status"] == "success"){
-                    $ipmi_addr = $ret["data"];
-                    if($do_update == "yes"){
-                        if($machine["product_name"] == "Lenovo" || $machine["product_name"] == "LENOVO"){
-                            # This produces a 16 char long pass
-                            $num_bytes = 12;
-                        }else{
-                            # This produces a 12 char long pass
-                            $num_bytes = 9;
+                    $ret = get_ip_of_machine_on_network($con, $conf, $matched_network["id"], $machine_id);
+                    if($ret["status"] != "success"){
+                        # If we don't have an IP on this network, there's 2 cases:
+                        # 1/ the machine never had its IPMI set and recorded in the DB.
+                        # 2/ the machine already was in an IPMI network, but that's the wrong one.
+                        # We simply get rid of all IPMI IPs for that machine, so we get back on case 1/.
+                        $qcheckwrong = "SELECT * FROM ips WHERE machine='".$machine["id"]."' AND usefor='ipmi'";
+                        $rcheckwrong = mysqli_query($con, $qcheckwrong);
+                        $ncheckwrong = mysqli_num_rows($rcheckwrong);
+                        if($ncheckwrong >= 0){
+                            $q_remove_ip = "DELETE FROM ips WHERE machine='".$machine["id"]."' AND usefor='ipmi'";
+                            $r_remove_ip = mysqli_query($con, $q_remove_ip);
+                        }
+
+                        $ret = reserve_ip_address($con, $conf, $net_id, $machine["id"], "ipmi");
+                        if($ret["status"] != "success"){
+                            print($ret["message"]);
+                        }
+                        $ret = get_ip_of_machine_on_network($con, $conf, $matched_network["id"], $machine_id);
+                        # Since we had to reserve a new IP, this means we got to do the IPMI setup.
+                        $do_update = "yes";
+                    }else{
+                        # As an IP address was already reserved, it means we don't need to do the setup.
+                        $do_update = "no";
+                    }
+                    if($ret["status"] == "success"){
+                        $ipmi_addr = $ret["data"];
+                        if($do_update == "yes"){
+                            if($machine["product_name"] == "Lenovo" || $machine["product_name"] == "LENOVO"){
+                                # This produces a 16 char long pass
+                                $num_bytes = 12;
+                            }else{
+                                # This produces a 12 char long pass
+                                $num_bytes = 9;
+                            }
+                            $hex = base64_encode(openssl_random_pseudo_bytes($num_bytes, $crypto_strong));
+                            $json = perform_ipmitool_cmd($con, $conf, $machine_id, $conf["ipmi"]["automatic_ipmi_username"], $hex, $ipmi_addr, $ipmi_net_def_gateway, $ipmi_netmask);
+                            $qmachine = "UPDATE machines SET ipmi_use='yes', ipmi_call_chassis_bootdev='yes', ipmi_addr='$ipmi_addr', ipmi_netmask='$ipmi_netmask', ipmi_default_gw='$ipmi_net_def_gateway', ipmi_vlan='$net_vlan', ipmi_username='". $conf["ipmi"]["automatic_ipmi_username"] ."', ipmi_password='$hex' WHERE id='$machine_id'";
+                            mysqli_query($con, $qmachine);
                         }
-                        $hex = base64_encode(openssl_random_pseudo_bytes($num_bytes, $crypto_strong));
-                        $json = perform_ipmitool_cmd($con, $conf, $machine_id, $conf["ipmi"]["automatic_ipmi_username"], $hex, $ipmi_addr, $ipmi_net_def_gateway, $ipmi_netmask);
-                        $qmachine = "UPDATE machines SET ipmi_use='yes', ipmi_call_chassis_bootdev='yes', ipmi_addr='$ipmi_addr', ipmi_netmask='$ipmi_netmask', ipmi_default_gw='$ipmi_net_def_gateway', ipmi_vlan='$net_vlan', ipmi_username='". $conf["ipmi"]["automatic_ipmi_username"] ."', ipmi_password='$hex' WHERE id='$machine_id'";
-                        mysqli_query($con, $qmachine);
                     }
-# This looks wrong: we should never get int his case.
-#                }else{
-#                    $qmachine = "UPDATE machines SET ipmi_use='yes', ipmi_call_chassis_bootdev='yes', ipmi_addr='$ipmi_addr', ipmi_netmask='$ipmi_netmask', ipmi_vlan='$net_vlan', ipmi_username='". $conf["ipmi"]["automatic_ipmi_username"] ."', ipmi_password='$hex' WHERE id='$machine_id'";
-#                    mysqli_query($con, $qmachine);
                 }
             }
 
