@@ -112,11 +112,28 @@ function safe_ip_list($param_name){
     }
     $safe_value = $_REQUEST[$param_name];
     foreach(explode(";", $safe_value) as $input_ip) {
-        if(strpos($input_ip, ':') !== FALSE) {
-            if(validate_ipv6_str($input_ip) == FALSE){ $safe_value = FALSE; }
-         } else {
-            if(validate_ipv4_str($input_ip) == FALSE){ $safe_value = FALSE; }
-         }
+        if(strpos($input_ip, '/') !== FALSE) {
+            $input_ip_ar = explode("/", $input_ip);
+            if(count($input_ip_ar) != 2) return FALSE;
+            $input_addr = $input_ip_ar[0];
+            $input_cidr = $input_ip_ar[1];
+            $reg = '/^[0-9]{1,253}$/';
+            if(!preg_match($reg,$input_cidr)) return FALSE;
+
+            if(strpos($input_addr, ':') !== FALSE) {
+                if($input_cidr < 0 || $input_cidr > 128) return FALSE;
+                if(validate_ipv6_str($input_addr) == FALSE){ $safe_value = FALSE; }
+            }else{
+                if($input_cidr < 0 || $input_cidr > 32) return FALSE;
+                if(validate_ipv4_str($input_addr) == FALSE){ $safe_value = FALSE; }
+            }
+        }else{
+            if(strpos($input_ip, ':') !== FALSE) {
+                if(validate_ipv6_str($input_ip) == FALSE){ $safe_value = FALSE; }
+            } else {
+                if(validate_ipv4_str($input_ip) == FALSE){ $safe_value = FALSE; }
+            }
+        }
     }
     return $safe_value;
 }
