@@ -299,7 +299,7 @@ if($n == 0){
             if(isset($iface["name"]) && isset($iface["macaddr"]) && isset($iface["max_speed"])){
                 $if_name = $iface["name"];
                 $reg = '/^[0-9a-zA-Z-]{1,64}$/';
-                if(!preg_match($reg,$blk_name)) die("Network interface device name suspicious line ".__LINE__." file ".__FILE__);
+                if(!preg_match($reg,$if_name)) die("Network interface device name suspicious line ".__LINE__." file ".__FILE__);
                 $safe_if_name = $if_name;
 
                 $if_macaddr = $iface["macaddr"];
@@ -324,7 +324,7 @@ if($n == 0){
 
                 if(isset($iface["firmware_version"]) && $iface["firmware_version"] != ""){
                     $if_firmware_version = $iface["firmware_version"];
-                    $reg = '/^[.:0-9a-zA-Z-]{1,64}$/';
+                    $reg = '/^[.:0-9a-zA-Z-+]{1,64}$/';
                     if(!preg_match($reg,$if_firmware_version)) die("Iface firmware version suspicious line ".__LINE__." file ".__FILE__);
                     $safe_if_firmware_version = $if_firmware_version;
                 }else{
@@ -610,7 +610,7 @@ if($n == 0){
 
                 if(isset($iface["firmware_version"]) && $iface["firmware_version"] != ""){
                     $if_firmware_version = $iface["firmware_version"];
-                    $reg = '/^[\/.:0-9a-zA-Z-]{1,64}$/';
+                    $reg = '/^[\/.:0-9a-zA-Z-+]{1,64}$/';
                     if(!preg_match($reg,$if_firmware_version)) die("Iface firmware version '$if_firmware_version' suspicious line ".__LINE__." file ".__FILE__);
                     $safe_if_firmware_version = $if_firmware_version;
                 }else{
